#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os, getpass
import dbus
import dbus.service
import pyinotify
import logging
from PyQt4 import QtCore
from threading import Thread, Event

from ldap_user import LdapUser


class PillBugClient(QtCore.QObject):
    sync_started_signal = QtCore.pyqtSignal()
    quota_data_signal = QtCore.pyqtSignal(str, str)
    status_signal = QtCore.pyqtSignal(int)
    progress_signal = QtCore.pyqtSignal(int, int)

    def __init__(self):
        QtCore.QObject.__init__(self, None)
        self.__bus = dbus.SystemBus()

        prx = self.__bus.get_object('org.cga.ticbox.pillbug', '/org/cga/ticbox/pillbug')
        self.pillbull_service = dbus.Interface(prx, dbus_interface='org.cga.ticbox.pillbug')

        self.dbus_signals = DBusSignals('/org/cga/ticbox/pillbug')
        self.__bus.add_signal_receiver(self.status_handler,
                                'sync_status', 
                                'org.cga.ticbox.pillbug')
        self.__bus.add_signal_receiver(self.progress_handler,
                                'sync_progress', 
                                'org.cga.ticbox.pillbug')

        self.__in_progress = False
        
        self.user = {}
        self.user['name'] = getpass.getuser()
        self.user['uid'] = os.getuid()
        self.user['gid'] = os.getgid()
        
        self.luser = LdapUser(self.user['name'])
        self.user['home'] = self.luser.get_home_dir()
        
        #Pyinotify file monitor...
        wm1 = pyinotify.WatchManager()
        s1 = pyinotify.Stats() # Stats is a subclass of ProcessEvent
        
        file_monitor = FileMonitor(s1)
        file_monitor.sync_monitor_signal.connect(self.sync_data)
        
        notifier1 = pyinotify.ThreadedNotifier(wm1, default_proc_fun=file_monitor)
        notifier1.start()
        
        monitor_dir = "%s/Documentos" % self.get_home_dir()
        logging.debug("Monitoring: %s" % monitor_dir)
        if not os.path.exists(monitor_dir):
            os.mkdir(monitor_dir)
        
        flags_monitor = pyinotify.IN_CLOSE_WRITE | pyinotify.IN_DELETE | pyinotify.IN_MOVED_FROM | pyinotify.IN_MOVED_TO | pyinotify.IN_CREATE
        wm1.add_watch(monitor_dir, flags_monitor, rec=True, auto_add=True)
        
        #Además, lanzar cada 5 min la actualización...
        self.sync_period = PeriodicSignal(300, file_monitor.sync_monitor_signal)
        self.sync_period.start()

    def is_movil_user(self):
        self.luser = LdapUser(self.user['name'])
        return self.luser.is_movil()
    
    def set_user_migration(self):
        self.luser.set_user_migration()

    def get_home_dir(self):
        return self.user['home']

    def signal_registers(self, get_user_data, initial_sync_data, sync_data, set_migration):
        get_user_data.connect(self.get_user_data)
        initial_sync_data.connect(self.initial_sync_data)
        sync_data.connect(self.sync_data)
        set_migration.connect(self.set_user_migration)

    def get_user_data(self):
        logging.debug("   PillBugClient => get_user_data")
        quota = self.pillbull_service.get_user_data(self.user['name'], self.user['uid'], self.user['gid'], self.user['home'])
        if quota[1] > 0:
            self.quota_data_signal.emit("%d" % quota[0], "%d" % quota[1])
        else:
            self.status_signal.emit(1) #Error, no hay cuota!

    def initial_sync_data(self):
        if not self.__in_progress:
            self.__in_progress = True
            logging.debug("   PillBugClient => initial_sync_data")
            self.dbus_signals.initial_sync_data(self.user['name'], self.user['uid'], self.user['gid'], self.user['home'])
            self.sync_started_signal.emit()

    def sync_data(self):
        if not self.__in_progress:
            self.__in_progress = True
            logging.info("Iniciando sincronización.")
            self.dbus_signals.sync_data(self.user['name'], self.user['uid'], self.user['gid'], self.user['home'])
            self.sync_started_signal.emit()

    def status_handler(self, uid, status):
        logging.debug("   PillBugClient => status_handler(%d)" % status)
        if uid == self.user['uid']:
            self.__in_progress = False
            self.status_signal.emit(status)
            
    def progress_handler(self, uid, done, total):
        logging.debug("   PillBugClient => progress_handler(%d / %d)" % (done, total) )
        if uid == self.user['uid']:
            self.progress_signal.emit(done, total)


class DBusSignals(dbus.service.Object):
    def __init__(self, object_path):
        dbus.service.Object.__init__(self, dbus.SystemBus(), object_path)

    @dbus.service.signal(dbus_interface='org.cga.ticbox.pillbug',
                         signature='suus')
    def initial_sync_data(self, user_name, uid, gid, home_dir):
        logging.debug("Emitiendo dbus -> initial_sync_data: sync_data")

    @dbus.service.signal(dbus_interface='org.cga.ticbox.pillbug',
                         signature='suus')
    def sync_data(self, user_name, uid, gid, home_dir):
        logging.debug("Emitiendo dbus -> sync_data: sync_data")

    
class FileMonitor(pyinotify.ProcessEvent, QtCore.QObject):
    sync_monitor_signal = QtCore.pyqtSignal()
    
    def __init__(self, stats):
        QtCore.QObject.__init__(self, None)
        super(type).__init__(type(stats))
        self.retarded = RetardedProccess(4, self.sync_monitor_signal)
    
    def process_default(self, event):
        # Si recibimos una ristra de cambios en el directorio, 
        # debemos esperarnos a la última señal del sync_monitor para lanzar self.sync_data...
        if not event.name.startswith('.') and not event.name.endswith('~') and event.name != '':
            logging.debug('Monitor (is_dir: %s): %s -> %s' % (event.dir, event.maskname, event.name))
            self.retarded.stop()
            self.retarded = RetardedProccess(3, self.sync_monitor_signal)
            self.retarded.start()


class PeriodicSignal(Thread):
    
    def __init__(self, interval, signal):
        Thread.__init__(self)
        self.interval = interval
        self.signal = signal
        self.finished = Event()

    def run(self):
        while not self.finished.is_set():
            self.finished.wait(self.interval)
            if not self.finished.is_set():
                self.signal.emit()

    def stop(self):
        self.finished.set()
        
class RetardedProccess(Thread):
    
    def __init__(self, interval, signal):
        Thread.__init__(self)
        self.interval = interval
        self.signal = signal
        self.finished = Event()

    def run(self):
        self.finished.wait(self.interval)
        if not self.finished.is_set():
            self.signal.emit()

    def stop(self):
        self.finished.set()